#
# accent.pl	--- translate dead-key accents to troff, tex etc. (library)
#
# Copyright (c) 1995 Oscar Nierstrasz
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program (as the file COPYING in the main directory of
# the distribution); if not, write to the Free Software Foundation,
# Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#
# This package and friends can be found at:
# http://www.iam.unibe.ch/~scg/Src/
#
# Filter "dead-key" conventions to produce the right
# sequences for TeX, Framemaker mml files or troff.
#
# Examples:
# acute accent:	\'a
# grave accent:	\`e
# circumflex:	\^o (or \<o)
# dieresis:	\"u (or \:u)
# tilde:	\~n
# cedilla:	\,c
# slash:	\/o
#
# ring:		\oa		*** FRAME ONLY
# ae:		\ae		*** FRAME ONLY
#
# tau:		\tau		*** PRESENTLY ONLY FOR TEX ***
# pi:		\pi
#
# UPDATES
# 21.11.97 -- on fixed nasty perl5 bug in &tex

# NB: if you copy this file by ftp BE SURE to use binary mode.
#
# THIS FILE CONTAINS 8-BIT CHARS!!!

package accent;

#v = 'accent.pl 1.0'; # Author: Oscar Nierstrasz (Revised: Jan 1990)
#v = 'accent.pl 1.1'; # Added 7 & 8 bit option (Sept 1991)
$v = 'accent.pl 1.2'; # Rewritten as perl script 27/6/93

# strip dead-key accents
sub strip {
    study;
    s/\\[:"]oe/oe/g;
    s/\\[:"]([AOUaou])/$1e/g;
    s/\\\/([Oo])/$1e/g;
    s/\\o([Aa])/$1a/g;
    s/\\ae/ae/g;
    s/\\ss/ss/g;
    s/\\alpha/alpha/g;
    s/\\beta/beta/g;
    s/\\pi/pi/g;
    s/\\tau/tau/g;
    s/\\.//g;
}

# convert dead-key accents to troff -ms .AM macros
sub ms {
    study;
    s/\\'([aeiou])/$1\\*'/g;
    s/\\`([aeiou])/$1\\*`/g;
    s/\\[\^<]([aeiou])/$1\\*^/g;
    s/\\:([aeiou])/$1\\*:/g;
    s/\\"([aeiou])/$1\\*:/g;
    s/\\:([AOU])/$1e/g;
    s/\\"([AOU])/$1e/g;
    s/\\~([aon])/$1\\*~/g;
    s/\\,([c])/$1\\*,/g;
    s/\\\/([Oo])/$1\\*\//g;
    s/\\ss/ss/g;
    s/\\alpha/alpha/g;
    s/\\beta/beta/g;
    s/\\tau/tau/g;
    s/\\pi/pi/g;
}

# convert dead-key accents to LaTeX
sub tex {
    study;
    s/\$/\\\$/g;
    s/[&#_]/\\$&/g;
    s/\\\/=/\$\\neq\$/g;
    # s/\\(['`^"])([aeou])/\\$1{$2}/g; # wrong $1{$2} is a hash context
    s/\\(['`^"])([aeou])/\\$1\{$2\}/g; # fixed on 21.11.97
    s/\\<([aeou])/\\^{$1}/g;
    s/\\(['`^"])i/\\$1{\\i}/g; # wrong
    s/\\(['`^"])i/\\$1\{\\i\}/g; # fixed on 21.11.97
    s/\\<i/\\^{\\i}/g;
    s/\\:([aeou])/\\"{$1}/g;
    s/\\:i/\\"{\\i}/g;
    s/\\~([aon])/\\~{$1}/g;
    s/\\,([c])/\\c{$1}/g;
    s/\\\/([Oo])/{\\o}/g;
    s/\\oA/\\AA/g;
    s/\\oa/\\aa/g;
    # s/\\AE/&/g
    # s/\\ae/&/g
    # s/\\ss/&/g
    s/\\alpha/\$$&\$/g;
    s/\\beta/\$$&\$/g;
    s/\\mu/\$$&\$/g;
    s/\\tau/\$$&\$/g;
    s/\\pi/\$$&\$/g;
    s/[<>=|]+/\$$&\$/g;
}

# convert tex charset to dead-key
# this was adapted from &tex by Matthias Rieger, February 18/22 2000
sub tex2deadkey {
    study;
    s/\\\$/\$/g;
    s/\\[&#_]/$&/g;
    s/\$\\neq\$/\\\/=/g;
    s/\\(['`^"])\{?([aeou])\}?/\\$1$2/g;
    s/\\^\{?([aeou])\}?/\\<$1/g;
    s/\\(['`^"])\{?\\i\}?/\\$1i/g;
    s/\\^\{?\\i\}?/\\<i/g;
    s/\\"\{?([aeou])\}?/\\:$1/g;
    s/\\"\{?\\i\}?/\\:i/g;
    s/\\~\{?([aon])\}?/\\~$1/g;
    s/\\c\{?c\}?/\\,c/g;
    s/\{?\\o\}?/\\\/o/g;
    s/\\AA/\\oA/g;
    s/\\aa/\\oa/g;

    s/\$(\\alpha)\$/$1/g;
    s/\$(\\beta)\$/$1/g;
    s/\$(\\mu)\$/$1/g;
    s/\$(\\tau)\$/$1/g;
    s/\$(\\pi)\$/$1/g;
    s/\$([<>=|]+)\$/$1/g;

    #remove all curly braces still in the text
    s/[{}]//g;
}

sub mml_init {
	print <<'eof' ;
<MML 1.0 Accent macros>
<Comment "copied from Oscar Nierstrasz, corrected and completed by E. Casais">
<Comment "accent.mml --- Framemaker accents">
<!DefineMacro UAgrave       "<Character \\xcb >" >
<!DefineMacro UAacute       "<Character \\xe7 >" >
<!DefineMacro UAcircumflex  "<Character \\xe5 >" >
<!DefineMacro UAtilde       "<Character \\xcc >" >
<!DefineMacro UAdieresis    "<Character \\x80 >" >
<!DefineMacro UAring        "<Character \\x81 >" >
<!DefineMacro UAE           "<Character \\xae >" >
<!DefineMacro agrave        "<Character \\x88 >" >
<!DefineMacro aacute        "<Character \\x87 >" >
<!DefineMacro acircumflex   "<Character \\x89 >" >
<!DefineMacro atilde        "<Character \\x8b >" >
<!DefineMacro adieresis     "<Character \\x8a >" >
<!DefineMacro aring         "<Character \\x8c >" >
<!DefineMacro ae            "<Character \\xbe >" >
<!DefineMacro UEgrave       "<Character \\xe9 >" >
<!DefineMacro UEacute       "<Character \\x83 >" >
<!DefineMacro UEcircumflex  "<Character \\xe6 >" >
<!DefineMacro UEdieresis    "<Character \\xe8 >" >
<!DefineMacro egrave        "<Character \\x8f >" >
<!DefineMacro eacute        "<Character \\x8e >" >
<!DefineMacro ecircumflex   "<Character \\x90 >" >
<!DefineMacro edieresis     "<Character \\x91 >" >
<!DefineMacro UIgrave       "<Character \\xed >" >
<!DefineMacro UIacute       "<Character \\xea >" >
<!DefineMacro UIcircumflex  "<Character \\xeb >" >
<!DefineMacro UIdieresis    "<Character \\xec >" >
<!DefineMacro igrave        "<Character \\x93 >" >
<!DefineMacro iacute        "<Character \\x92 >" >
<!DefineMacro icircumflex   "<Character \\x94 >" >
<!DefineMacro idieresis     "<Character \\x95 >" >
<!DefineMacro dotlessi      "<Character \\xf5 >" >
<!DefineMacro UOgrave       "<Character \\xf1 >" >
<!DefineMacro UOacute       "<Character \\xee >" >
<!DefineMacro UOcircumflex  "<Character \\xef >" >
<!DefineMacro UOtilde       "<Character \\xcd >" >
<!DefineMacro UOdieresis    "<Character \\x85 >" >
<!DefineMacro UOslash       "<Character \\xaf >" >
<!DefineMacro UOE           "<Character \\xce >" >
<!DefineMacro ograve        "<Character \\x98 >" >
<!DefineMacro oacute        "<Character \\x97 >" >
<!DefineMacro ocircumflex   "<Character \\x99 >" >
<!DefineMacro otilde        "<Character \\x9b >" >
<!DefineMacro odieresis     "<Character \\x9a >" >
<!DefineMacro oslash        "<Character \\xbf >" >
<!DefineMacro oe            "<Character \\xcf >" >
<!DefineMacro UUgrave       "<Character \\xf4 >" >
<!DefineMacro UUacute       "<Character \\xf2 >" >
<!DefineMacro UUcircumflex  "<Character \\xf3 >" >
<!DefineMacro UUdieresis    "<Character \\x86 >" >
<!DefineMacro ugrave        "<Character \\x9d >" >
<!DefineMacro uacute        "<Character \\x9c >" >
<!DefineMacro ucircumflex   "<Character \\x9e >" >
<!DefineMacro udieresis     "<Character \\x9f >" >
<!DefineMacro UYdieresis    "<Character \\xd9 >" >
<!DefineMacro ydieresis     "<Character \\xd8 >" >
<!DefineMacro UCcedilla     "<Character \\x82 >" >
<!DefineMacro ccedilla      "<Character \\x8d >" >
<!DefineMacro UNtilde       "<Character \\x84 >" >
<!DefineMacro ntilde        "<Character \\x96 >" >
<!DefineMacro germandbls    "<Character \\xa7 >" >
<!DefineMacro alpha      "<family symbol> <Character \\x61 > <family times>" >
<!DefineMacro beta       "<family symbol> <Character \\x62 > <family times>" >
<!DefineMacro pi         "<family symbol> <Character \\x70 > <family times>" >
<!DefineMacro tau        "<family symbol> <Character \\x74 > <family times>" >
<!DefineMacro endash        "<Character \\xd0 >" >
<!DefineMacro emdash        "<Character \\xd1 >" >
<!DefineMacro quotedblleft  "<Character \\xd2 >" >
<!DefineMacro quoteblright  "<Character \\xd3 >" >
<!DefineMacro quoteleft     "<Character \\xd4 >" >
<!DefineMacro quoteright    "<Character \\xd5 >" >
<!DefineMacro guillemotleft "<Character \\xc7 >" >
<!DefineMacro guillemotright "<Character \\xc8 >" >
<!DefineMacro guilsinglleft "<Character \\xdc >" >
<!DefineMacro guilsinglright "<Character \\xdd >" >
<!DefineMacro langle "<Character \\< >" >
<!DefineMacro rangle "<Character \\> >" >
eof
}

# convert dead-key accents to Framemaker MML
sub mml {
    study;
    s/\\'([aeiou])/<$1acute>/g;
    s/\\`([aeiou])/<$1grave>/g;
    s/\\[\^<]([aeiou])/<$1circumflex>/g;
    s/\\:([aeiou])/<$1dieresis>/g;
    s/\\"([aeiou])/<$1dieresis>/g;
    s/\\~([aon])/<$1tilde>/g;
    s/\\'([AEIOU])/<U$1acute>/g;
    s/\\`([AEIOU])/<U$1grave>/g;
    s/\\[\^<]([AEIOU])/<U$1circumflex>/g;
    s/\\:([AEIOU])/<U$1dieresis>/g;
    s/\\"([AEIOU])/<U$1dieresis>/g;
    s/\\~([AON])/<U$1tilde>/g;
    s/\\,([c])/<$1cedilla>/g;
    s/\\oA/<UAring>/g;
    s/\\oa/<aring>/g;
    s/\\AE/<UAE>/g;
    s/\\ae/<ae>/g;
    s/\\ss/<germandbls>/g;
    # s/\\alpha/<alpha>/g
    # s/\\beta/<beta>/g
    # s/\\tau/<tau>/g
    # s/\\pi/<pi>/g
    s/\\\/([Oo])/<$1slash>/g;
    s/``/<quotedblleft>/g;
    s/''/<quoteblright>/g;
    s/--/<emdash>/g;
}

# convert dead-key accents to HTML
sub html {
    study;
    s/\\AE/\&AElig;/g;
    s/\\'([AEIOUYaeiouy])/\&$1acute;/g;
    s/\\[<^]([AEIOUaeiou])/\&$1circ;/g;
    s/\\`([AEIOUaeiou])/\&$1grave;/g;
    s/\\o([Aa])/\&$1ring;/g;
    s/\\~([ANOano])/\&$1tilde;/g;
    s/\\[:"]([AEIOUYaeiouy])/\&$1uml;/g;
    s/\\,([Cc])/\&$1cedil;/g;
    s/\\\/([Oo])/\&$1slash;/g;
    s/\\ss/\&szlig;/g;
}

# convert dead-key accents to overstruck characters
sub os {
    study;
    s/\\'([aeiou])/\\o'\\'$1'/g;
    s/\\`([aeiou])/\\o'\\(ga$1'/g;
    s/\\[\^<]([aeiou])/\\o'^$1'/g;
    s/\\[:"]([aeiou])/\\o'\\(um$1'/g;
    s/\\~([an])/\\o'~$1'/g;
    s/\\,([c])/\\o'\\(cdc'/g;
    s/\\\/([Oo])/\\o'\/$1'/g;
    s/\\tau/tau/g;
    s/\\pi/pi/g;
}

# convert nixdorf accents to dead-key accents
sub nix {
    study;
    s/\\\*([aeiouAEIOU])/\\"$1/g;
    s/\\tau/tau/g;
    s/\\pi/pi/g;
}

# convert accents from PC (DOS) files
# used for SI files
sub filemaker {
    study;
    s/\210/\\`a/g;
    s/\212/\\:a/g;
    s/\215/\\,c/g;
    s/\216/\\'e/g;
    s/\217/\\`e/g;
    s/\220/\\<e/g;
    s/\230/\\`o/g;
    s/\231/\\<o/g;
    s/\232/\\:o/g;
    s/\237/\\:u/g;
}

# convert accents from PC (DOS) files
# used for SI files
# pipe through cat -v first
sub si {
    study;
    s/M-^H/\\`a/g; s/M-^J/\\:a/g; s/M-^M/\\,c/g;
    s/M-^N/\\'e/g; s/M-^O/\\`e/g; s/M-^P/\\<e/g;
    s/M-^Y/\\<a/g; s/M-^Z/\\:o/g; s/M-^_/\\:u/g;
}

# convert accents from PC (DOS) files
# pipe through cat -v first
# Used for Gert Florijn's address files
sub pc {
    study;
    s/M-^A/\\:u/g; s/M-^B/\\'e/g; s/M-^C/\\<a/g;
    s/M-^D/\\:a/g; s/M-^E/\\`a/g; s/M-^G/\\,c/g;
    s/M-^H/\\<e/g; s/M-^I/\\:e/g; s/M-^J/\\`e/g;
    s/M-^K/\\:i/g; s/M-^L/\\<i/g; s/M-^S/\\<o/g;
    s/M-^T/\\:o/g;
}

# convert from Mac (Word?) files
# INCOMPLETE
sub mac {
	s/=46/F/g;
	s/=C9/\\'E/g;
	s/=DC/\\:U/g;
	s/=E0/\\`a/g;
	s/=E2/\\<a/g;
	s/=E7/\\,c/g;
	s/=E8/\\`e/g;
	s/=E9/\\'e/g;
	s/=EA/\\<e/g;
	s/=EB/\\:e/g;
	s/=FC/\\:u/g;
        s/=E4/\\:a/g;
        s/=F4/\\<o/g;
        s/=F6/\\:o/g;
        s/=FC/\\:u/g;
        s/=93/``/g;
        s/=\n//g;
}

# convert 8bit ascii to 7bit escapes
#
# not handled:
#               
sub seven {
    study;
    s//'/g; s//``/g; s//''/g;
    s//\\bu/g;
    s//\\bu/g; s//\\??/g; s//\\!!/g; s//\\xx/g;
    s//\\+-/g; s//\\-:/g; s//\\<</g; s//\\>>/g;
    s//\\-D/g; s//\\-L/g; s//\\-Y/g; s//\\\/c/g;
    s//\\12/g; s//\\14/g; s//\\34/g; s//\\^1/g;
    s//\\^2/g; s//\\^3/g; s//\\ss/g; s//\\\/u/g;
    s//\\so/g; s//\\beta/g; s//\\mu/g; s//\\co/g;
    s//\\ro/g; s//\\AE/g; s//\\ae/g; s//\\oA/g;
    s//\\oa/g; s//\\\/O/g; s//\\\/o/g; s//\\,C/g;
    s//\\,c/g; s//\\`A/g; s//\\`E/g; s//\\`I/g;
    s//\\`O/g; s//\\`U/g; s//\\`a/g; s//\\`e/g;
    s//\\`i/g; s//\\`o/g; s//\\`u/g; s//\\'A/g;
    s//\\'E/g; s//\\'I/g; s//\\'O/g; s//\\'U/g;
    s//\\'Y/g; s//\\'a/g; s//\\'e/g; s//\\'i/g;
    s//\\'o/g; s//\\'u/g; s//\\'y/g; s//\\:A/g;
    s//\\:E/g; s//\\:I/g; s//\\:O/g; s//\\:U/g;
    s//\\:a/g; s//\\:e/g; s//\\:i/g; s//\\:o/g;
    s//\\:u/g; s//\\:y/g; s//\\"A/g; s//\\"E/g;
    s//\\"I/g; s//\\"O/g; s//\\"U/g; s//\\"a/g;
    s//\\"e/g; s//\\"i/g; s//\\"o/g; s//\\"u/g;
    s//\\"y/g; s//\\<A/g; s//\\<E/g; s//\\<I/g;
    s//\\<O/g; s//\\<U/g; s//\\<a/g; s//\\<e/g;
    s//\\<i/g; s//\\<o/g; s//\\<u/g; s//\\^A/g;
    s//\\^E/g; s//\\^I/g; s//\\^O/g; s//\\^U/g;
    s//\\^a/g; s//\\^e/g; s//\\^i/g; s//\\^o/g;
    s//\\^u/g; s//\\~A/g; s//\\~N/g; s//\\~O/g;
    s//\\~a/g; s//\\~n/g; s//\\~o/g;
}

# convert dead-key accents to 8bit meta-chars
# not handled:
#               
sub eight {
    study;
    s/\\bu//g; s/\\\?\?//g; s/\\!!//g; s/\\xx//g;
    s/\\\+-//g; s/\\-://g; s/\\<<//g; s/\\>>//g;
    s/\\-D//g; s/\\-L//g; s/\\-Y//g; s/\\\/c//g;
    s/\\12//g; s/\\14//g; s/\\34//g; s/\\\^1//g;
    s/\\\^2//g; s/\\\^3//g; s/\\ss//g; s/\\\/u//g;
    s/\\so//g; s/\\beta//g; s/\\mu//g; s/\\co//g;
    s/\\ro//g; s/\\AE//g; s/\\ae//g; s/\\oA//g;
    s/\\oa//g; s/\\\/O//g; s/\\\/o//g; s/\\,C//g;
    s/\\,c//g; s/\\`A//g; s/\\`E//g; s/\\`I//g;
    s/\\`O//g; s/\\`U//g; s/\\`a//g; s/\\`e//g;
    s/\\`i//g; s/\\`o//g; s/\\`u//g; s/\\'A//g;
    s/\\'E//g; s/\\'I//g; s/\\'O//g; s/\\'U//g;
    s/\\'Y//g; s/\\'a//g; s/\\'e//g; s/\\'i//g;
    s/\\'o//g; s/\\'u//g; s/\\'y//g; s/\\:A//g;
    s/\\:E//g; s/\\:I//g; s/\\:O//g; s/\\:U//g;
    s/\\:a//g; s/\\:e//g; s/\\:i//g; s/\\:o//g;
    s/\\:u//g; s/\\:y//g; s/\\"A//g; s/\\"E//g;
    s/\\"I//g; s/\\"O//g; s/\\"U//g; s/\\"a//g;
    s/\\"e//g; s/\\"i//g; s/\\"o//g; s/\\"u//g;
    s/\\"y//g; s/\\<A//g; s/\\<E//g; s/\\<I//g;
    s/\\<O//g; s/\\<U//g; s/\\<a//g; s/\\<e//g;
    s/\\<i//g; s/\\<o//g; s/\\<u//g; s/\\^A//g;
    s/\\^E//g; s/\\^I//g; s/\\^O//g; s/\\^U//g;
    s/\\^a//g; s/\\^e//g; s/\\^i//g; s/\\^o//g;
    s/\\^u//g; s/\\~A//g; s/\\~N//g; s/\\~O//g;
    s/\\~a//g; s/\\~n//g; s/\\~o//g;
}

1;

